@@ -18,6 +18,8 @@
 #include "linc-private.h"
 #include "linc-debug.h"
 
+#undef LOCAL_DEBUG
+
 static char linc_tmpdir [PATH_MAX] = "";
 
 /*
@@ -74,6 +76,8 @@ make_local_tmpdir (const char *dirname)
  * Set the temporary directory used by linc to @dir. 
  *
  * This directory is used for the creation of UNIX sockets.
+ * @dir must have the correct permissions, 0700, user owned
+ * otherwise this method will g_error.
  **/
 void
 linc_set_tmpdir (const char *dir)
@@ -85,6 +89,21 @@ linc_set_tmpdir (const char *dir)
 	make_local_tmpdir (linc_tmpdir);
 }
 
+/**
+ * linc_get_tmpdir:
+ * @void: 
+ * 
+ * Fetches the directory name used by linc to whack
+ * Unix Domain sockets into.
+ * 
+ * Return value: the g_allocated socket name.
+ **/
+char *
+linc_get_tmpdir (void)
+{
+	return g_strdup (linc_tmpdir);
+}
+
 #ifdef HAVE_SOCKADDR_SA_LEN
 #define LINC_SET_SOCKADDR_LEN(saddr, len)                     \
 		((struct sockaddr *)(saddr))->sa_len = (len)
@@ -94,15 +113,17 @@ linc_set_tmpdir (const char *dir)
 
 #if defined(AF_INET6) && defined(RES_USE_INET6)
 #define LINC_RESOLV_SET_IPV6     _res.options |= RES_USE_INET6
-#define LINC_RESOLV_CLEAR_IPV6   _res.options &= ~RES_USE_INET6
+#define LINC_RESOLV_UNSET_IPV6   _res.options &= ~RES_USE_INET6
 #else
 #define LINC_RESOLV_SET_IPV6
-#define LINC_RESOLV_CLEAR_IPV6
+#define LINC_RESOLV_UNSET_IPV6
 #endif
 
+
+
 #if defined(AF_INET) || defined(AF_INET6) || defined (AF_UNIX)
-static const char *
-get_local_hostname (void)
+const char *
+linc_get_local_hostname (void)
 {
 	static char local_host[NI_MAXHOST] = { 0 };
 
@@ -115,6 +136,40 @@ get_local_hostname (void)
 	return local_host;
 }
 
+/*
+ * True if succeeded in mapping, else false.
+ */
+static gboolean
+ipv4_addr_from_addr (struct in_addr *dest_addr,
+		     guint8         *src_addr,
+		     int             src_length)
+{
+	if (src_length == 4)
+		memcpy (dest_addr, src_addr, 4);
+
+	else if (src_length == 16) {
+		int i;
+
+#ifdef LOCAL_DEBUG
+		g_warning ("Doing conversion ...");
+#endif
+
+		/* An ipv6 address, might be an IPv4 mapped though */
+		for (i = 0; i < 10; i++)
+			if (src_addr [i] != 0)
+				return FALSE;
+
+		if (src_addr [10] != 0xff ||
+		    src_addr [11] != 0xff)
+			return FALSE;
+
+		memcpy (dest_addr, &src_addr[12], 4);
+	} else
+		return FALSE;
+
+	return TRUE;
+}
+
 static gboolean
 linc_protocol_is_local_ipv46 (const LINCProtocolInfo *proto,
 			      const struct sockaddr   *saddr,
@@ -122,34 +177,73 @@ linc_protocol_is_local_ipv46 (const LINCProtocolInfo *proto,
 {
 	int i;
 	static int warned = 0;
-	static struct hostent *local_hostent = NULL;
-	
-	if (!local_hostent)
-		local_hostent = gethostbyname (get_local_hostname ());
+	static struct hostent *local_hostent;
+
+	g_assert (saddr->sa_family == proto->family);
+
+	if (!local_hostent) {
+		LINC_RESOLV_SET_IPV6;
+		local_hostent = gethostbyname (linc_get_local_hostname ());
+	}
 
 	if (!local_hostent) {
 		if (!warned++)
 			g_warning ("can't gethostbyname on '%s'",
-				   get_local_hostname ());
+				   linc_get_local_hostname ());
 		return FALSE;
 	}
 
-	if (local_hostent->h_addrtype != saddr->sa_family) {
-		if (!warned++)
-			g_warning ("FIXME: can't compare different family "
-				   "address types for locality");
+	if (!local_hostent->h_addr_list)
+		g_error ("No address for local host");
+
+	if (proto->family != AF_INET) {
+#ifdef AF_INET6
+		if (proto->family == AF_INET6 &&
+		    local_hostent->h_addrtype != AF_INET6)
+			return FALSE; /* can connect via IPv4 */
+
+		if (proto->family != AF_INET6)
+			return FALSE;
+#else
 		return FALSE;
+#endif
 	}
 
-	g_warning ("Compare vs %d matches len %d",
-		   local_hostent->h_length, saddr_len);
+	for (i = 0; local_hostent->h_addr_list [i]; i++) {
 
-	for (i = 0; i < local_hostent->h_length; i++) {
-		if (!memcmp (local_hostent->h_addr_list [i],
-			     saddr->sa_data, saddr_len))
+		if (proto->family == AF_INET) {
+			struct in_addr ipv4_addr;
+			
+			if (!ipv4_addr_from_addr (&ipv4_addr,
+						  local_hostent->h_addr_list [i],
+						  local_hostent->h_length))
+				continue;
+
+			if (!memcmp (&ipv4_addr, 
+				     &((struct sockaddr_in *)saddr)->sin_addr.s_addr, 4)) {
+#ifdef LOCAL_DEBUG
+				g_warning ("local ipv4 address");
+#endif
+				return TRUE;
+			}
+
+		}
+#ifdef AF_INET6
+		else if (!memcmp (local_hostent->h_addr_list [i],
+				  &((struct sockaddr_in6 *)saddr)->sin6_addr.s6_addr,
+				  local_hostent->h_length)) {
+#ifdef LOCAL_DEBUG
+			g_warning ("local ipv6 address");
+#endif
 			return TRUE;
+		}
+#endif
 	}
 
+#ifdef LOCAL_DEBUG
+	g_warning ("No match over all");
+#endif
+
 	return FALSE;
 }
 
@@ -193,20 +287,32 @@ linc_protocol_get_sockaddr_ipv4 (const LINCProtocolInfo *proto,
 	saddr->sin_family = AF_INET;
 	saddr->sin_port   = htons (atoi (portnum));
 
-	if (!(_res.options & RES_INIT))
-		res_init();
+	if ((saddr->sin_addr.s_addr = inet_addr (hostname)) == INADDR_NONE) {
+	        int i;
 
-	LINC_RESOLV_CLEAR_IPV6;
+		LINC_RESOLV_UNSET_IPV6;
+		if (!(_res.options & RES_INIT))
+			res_init();
+		
+		host = gethostbyname (hostname);
+		if (!host) {
+		  g_free (saddr);
+		  return NULL;
+		}
 
-	host = gethostbyname (hostname);
-	if (!host) {
-		g_free (saddr);
-		return NULL;
+		for(i = 0; host->h_addr_list[i]; i++)
+		    if(ipv4_addr_from_addr (&saddr->sin_addr,
+					    (guint8 *)host->h_addr_list [i],
+					    host->h_length))
+		      break;
+
+		if(!host->h_addr_list[i]) {
+		  g_free (saddr);
+		  return NULL;
+		}
 	}
 
-	memcpy (&saddr->sin_addr, host->h_addr_list[0], sizeof (struct in_addr));
-
-	return (struct sockaddr *)saddr;
+	return (struct sockaddr *) saddr;
 }
 #endif /* AF_INET */
 
@@ -249,12 +355,15 @@ linc_protocol_get_sockaddr_ipv6 (const LINCProtocolInfo *proto,
 
 	saddr->sin6_family = AF_INET6;
 	saddr->sin6_port = htons (atoi (portnum));
+#ifdef HAVE_INET_PTON
+	if (inet_pton (AF_INET6, hostname, &saddr->sin6_addr) > 0)
+		return (struct sockaddr *)saddr;
+#endif
 
 	if (!(_res.options & RES_INIT))
 		res_init();
 
 	LINC_RESOLV_SET_IPV6;
-
 	host = gethostbyname (hostname);
 	if (!host || host->h_addrtype != AF_INET6) {
 		g_free (saddr);
@@ -268,58 +377,6 @@ linc_protocol_get_sockaddr_ipv6 (const LINCProtocolInfo *proto,
 #endif /* AF_INET6 */
 
 #ifdef AF_UNIX
-/*
- *   This is rumoured not to work at all
- * with 2.0.X kernels.
- */
-static void
-cleanup_sweep (void)
-{
-#if 0
-	int removed = 0, to_remove;
-	DIR *dirh;
-	struct dirent *dent;
-
-	dirh = opendir (linc_tmpdir);
-
-#ifdef G_ENABLE_DEBUG
-	to_remove = G_MAXINT;
-#else
-	to_remove = 4; /* We should create 1 */
-#endif
-
-	while (removed < to_remove && (dent = readdir (dirh))) {
-		int usfd, ret, saddr_len;
-		struct sockaddr_un saddr;
-
-		saddr.sun_family = AF_UNIX;
-
-		if (strncmp (dent->d_name, "linc-", 5))
-			continue;
-
-		g_snprintf (saddr.sun_path,
-			    sizeof (saddr.sun_path),
-			    "%s/%s", linc_tmpdir, dent->d_name);
-
-		usfd = socket (AF_UNIX, SOCK_STREAM, 0);
-		g_assert (usfd >= 0);
-
-		saddr_len = sizeof (struct sockaddr_un) -
-			sizeof (saddr.sun_path) + strlen (saddr.sun_path);
-
-		ret = connect (usfd, &saddr, saddr_len);
-		close (usfd);
-
-		if (ret >= 0)
-			continue;
-
-		unlink (saddr.sun_path);
-		removed++;
-	}
-	closedir (dirh);
-#endif
-}
-
 /*
  * linc_protocol_get_sockaddr_unix:
  * @proto: the #LINCProtocolInfo structure for the UNIX sockets protocol.
@@ -344,15 +401,9 @@ linc_protocol_get_sockaddr_unix (const LINCProtocolInfo *proto,
 	struct sockaddr_un *saddr;
 	int                 pathlen;
 	char                buf[64], *actual_path;
-	static gboolean     done_cleanup_sweep = FALSE;
 
 	g_assert (proto->family == AF_UNIX);
 
-	if (!done_cleanup_sweep) {
-		done_cleanup_sweep = TRUE;
-		cleanup_sweep ();
-	}
-
 	if (!path) {
 		struct timeval t;
 		static guint pid = 0, idx = 0;
@@ -362,15 +413,10 @@ linc_protocol_get_sockaddr_unix (const LINCProtocolInfo *proto,
 
 		gettimeofday (&t, NULL);
 		g_snprintf (buf, sizeof (buf),
-/* We'd do this all the time, but the pid is not very random,
- * and this string is truncated by some unixes */
-#ifdef CONNECTION_DEBUG
-			    "%s/l-%d-%x%x", linc_tmpdir, pid,
-#else
-			    "%s/linc-%x%x", linc_tmpdir,
-#endif
-			    (guint) (rand() ^ t.tv_sec ^ pid),
-			    (guint) (idx++ ^ t.tv_usec));
+			    "%s/linc-%x-%x-%x%x", linc_tmpdir,
+			    pid, idx++,
+			    (guint) (rand() ^ t.tv_sec),
+			    (guint) (idx ^ t.tv_usec));
 #ifdef CONNECTION_DEBUG
 		if (g_file_test (buf, G_FILE_TEST_EXISTS))
 			g_warning ("'%s' already exists !", buf);
@@ -475,9 +521,10 @@ linc_protocol_get_sockinfo_ipv46 (struct hostent  *host,
 	if (!host) {
 		const char *local_host;
 
-		if (!(local_host = get_local_hostname ()))
+		if (!(local_host = linc_get_local_hostname ()))
 			return FALSE;
 
+		LINC_RESOLV_SET_IPV6;
 		host = gethostbyname (local_host);
 	}
 
@@ -490,7 +537,7 @@ linc_protocol_get_sockinfo_ipv46 (struct hostent  *host,
 	if (portnum) {
 		gchar tmpport[NI_MAXSERV];
 
-		g_snprintf (tmpport, NI_MAXSERV, "%d", ntohs (port));
+		g_snprintf (tmpport, sizeof (tmpport), "%d", ntohs (port));
 
 		*portnum = g_strdup (tmpport);
 	}
@@ -620,7 +667,7 @@ linc_protocol_get_sockinfo_unix (const LINCProtocolInfo  *proto,
 	if (hostname) {
 		const char *local_host;
 
-		if (!(local_host = get_local_hostname ()))
+		if (!(local_host = linc_get_local_hostname ()))
 			return FALSE;
 
 		*hostname = g_strdup (local_host);
@@ -880,7 +927,7 @@ irda_find_device (guint32  *addr,
 	}
 
  out:
-	close(fd);
+	LINC_CLOSE (fd);
 
 	return retval;
 }
@@ -927,7 +974,7 @@ irda_getaddrinfo (const char             *nodename,
 				return EAI_NONAME;
 
 			/* It's a numeric address - we need to find the hostname */
-			strcpy (hnbuf, IRDA_PREFIX);
+			g_strncpy (hnbuf, IRDA_PREFIX, IRDA_NICKNAME_MAX + IRDA_PREFIX_LEN);
 			if (irda_find_device (&sai.sir_addr, 
 					      hnbuf + IRDA_PREFIX_LEN,
 					      FALSE))
@@ -961,7 +1008,7 @@ irda_getaddrinfo (const char             *nodename,
 	retval->ai_addr = (struct sockaddr *)tptr;
 	memcpy (retval->ai_addr, &sai, sizeof (struct sockaddr_irda));
 	tptr += sizeof (struct sockaddr_irda);
-	strcpy (tptr, hnbuf);
+	g_strncpy (tptr, hnbuf, IRDA_NICKNAME_MAX + IRDA_PREFIX_LEN);
 	retval->ai_family = AF_IRDA;
 	retval->ai_socktype = SOCK_STREAM;
 	retval->ai_protocol = 0;
@@ -1026,7 +1073,7 @@ linc_protocol_destroy_cnx (const LINCProtocolInfo *proto,
 		if (proto->destroy)
 			proto->destroy (fd, host, service);
 		
-		close (fd);
+		LINC_CLOSE (fd);
 	}
 }
 
@@ -1047,7 +1094,7 @@ linc_protocol_destroy_addr (const LINCProtocolInfo *proto,
 			proto->destroy (fd, NULL, addr_un->sun_path);
 		}
 #endif
-		close (fd);
+		LINC_CLOSE (fd);
 		g_free (saddr);
 	}
 
